@@ -55,9 +55,7 @@ class TwoStageFallbackAction(LoopAction):
         domain: Domain,
     ) -> List[Event]:
         affirm_action = action.action_from_name(
-            ACTION_DEFAULT_ASK_AFFIRMATION_NAME,
-            self._action_endpoint,
-            domain.user_actions,
+            ACTION_DEFAULT_ASK_AFFIRMATION_NAME, domain, self._action_endpoint
         )
 
         return await affirm_action.run(output_channel, nlg, tracker, domain)
@@ -70,7 +68,7 @@ class TwoStageFallbackAction(LoopAction):
         domain: Domain,
     ) -> List[Event]:
         rephrase = action.action_from_name(
-            ACTION_DEFAULT_ASK_REPHRASE_NAME, self._action_endpoint, domain.user_actions
+            ACTION_DEFAULT_ASK_REPHRASE_NAME, domain, self._action_endpoint
         )
 
         return await rephrase.run(output_channel, nlg, tracker, domain)
@@ -115,7 +113,7 @@ class TwoStageFallbackAction(LoopAction):
         domain: Domain,
     ) -> List[Event]:
         fallback = action.action_from_name(
-            ACTION_DEFAULT_FALLBACK_NAME, self._action_endpoint, domain.user_actions
+            ACTION_DEFAULT_FALLBACK_NAME, domain, self._action_endpoint
         )
 
         return await fallback.run(output_channel, nlg, tracker, domain)
