@@ -10,6 +10,7 @@ from typing import Any, Awaitable, Callable, Dict, List, Optional, Text
 
 from rasa.core.channels.channel import InputChannel, OutputChannel, UserMessage
 from rasa.shared.constants import DOCS_URL_CONNECTORS_SLACK
+from rasa.shared.exceptions import InvalidConfigException, RasaException
 import rasa.shared.utils.io
 from sanic import Blueprint, response
 from sanic.request import Request
@@ -134,7 +135,6 @@ class SlackInput(InputChannel):
         if not credentials:
             cls.raise_missing_credentials_exception()
 
-        # pytype: disable=attribute-error
         return cls(
             credentials.get("slack_token"),
             credentials.get("slack_channel"),
@@ -143,9 +143,8 @@ class SlackInput(InputChannel):
             credentials.get("slack_retry_number_header", "x-slack-retry-num"),
             credentials.get("errors_ignore_retry", None),
             credentials.get("use_threads", False),
-            credentials.get("slack_signing_secret", None),
+            credentials.get("slack_signing_secret", ""),
         )
-        # pytype: enable=attribute-error
 
     def __init__(
         self,
@@ -156,7 +155,7 @@ class SlackInput(InputChannel):
         slack_retry_number_header: Optional[Text] = None,
         errors_ignore_retry: Optional[List[Text]] = None,
         use_threads: Optional[bool] = False,
-        slack_signing_secret: Optional[Text] = None,
+        slack_signing_secret: Text = "",
     ) -> None:
         """Create a Slack input channel.
 
@@ -199,18 +198,17 @@ class SlackInput(InputChannel):
         self.use_threads = use_threads
         self.slack_signing_secret = slack_signing_secret
 
-        self._raise_deprecation_warnings()
+        self._validate_credentials()
 
-    def _raise_deprecation_warnings(self) -> None:
-        """Raises any deprecation warning regarding configuration parameters."""
-        if self.slack_signing_secret is None:
-            rasa.shared.utils.io.raise_deprecation_warning(
-                "Your slack bot is missing a configured signing secret. Running a "
-                "bot without a signing secret is deprecated and will be removed in "
-                "the next release (2.2.0). You should add a `slack_signing_secret` "
-                "parameter to your channel configuration.",
-                warn_until_version="2.2.0",
-                docs=DOCS_URL_CONNECTORS_SLACK,
+    def _validate_credentials(self) -> None:
+        """Raises exceptions if the connector is not properly configured."""
+        if not self.slack_signing_secret:
+            raise InvalidConfigException(
+                f"Your slack bot is missing a configured signing secret. Running a "
+                f"bot without a signing secret is insecure and was removed. "
+                f"You need to add a `slack_signing_secret` parameter to your channel "
+                f"configuration. "
+                f"More info at {DOCS_URL_CONNECTORS_SLACK} ."
             )
 
     @staticmethod
@@ -403,10 +401,19 @@ class SlackInput(InputChannel):
             event = slack_event.get("event", {})
             thread_id = event.get("thread_ts", event.get("ts"))
 
+            users = []
+            if "authed_users" in slack_event:
+                users = slack_event.get("authed_users")
+            elif (
+                "authorizations" in slack_event
+                and len(slack_event.get("authorizations")) > 0
+            ):
+                users.append(slack_event.get("authorizations")[0].get("user_id"))
+
             return {
                 "out_channel": event.get("channel"),
                 "thread_id": thread_id,
-                "users": slack_event.get("authed_users"),
+                "users": users,
             }
 
         if content_type == "application/x-www-form-urlencoded":
@@ -415,10 +422,15 @@ class SlackInput(InputChannel):
             payload = json.loads(output["payload"][0])
             message = payload.get("message", {})
             thread_id = message.get("thread_ts", message.get("ts"))
+
+            users = []
+            if payload.get("user", {}).get("id"):
+                users.append(payload.get("user", {}).get("id"))
+
             return {
                 "out_channel": payload.get("channel", {}).get("id"),
                 "thread_id": thread_id,
-                "users": payload.get("user", {}).get("id"),
+                "users": users,
             }
 
         return {}
@@ -435,10 +447,6 @@ class SlackInput(InputChannel):
         Returns:
             `True` if the request came from Slack.
         """
-        if self.slack_signing_secret is None:
-            # this is done for backwards compatibility, but will be removed in
-            # the next release
-            return True
 
         try:
             slack_signing_secret = bytes(self.slack_signing_secret, "utf-8")
