@@ -9,13 +9,11 @@ from rasa.shared.exceptions import RasaException
 import rasa.shared.utils.io
 import rasa.utils.io
 from rasa.constants import MINIMUM_COMPATIBLE_VERSION, NLU_MODEL_NAME_PREFIX
-from rasa.nlu import components, utils  # pytype: disable=pyi-error
-from rasa.nlu.classifiers.classifier import (  # pytype: disable=pyi-error
-    IntentClassifier,
-)
-from rasa.nlu.components import Component, ComponentBuilder  # pytype: disable=pyi-error
+from rasa.nlu import components, utils
+from rasa.nlu.classifiers.classifier import IntentClassifier
+from rasa.nlu.components import Component, ComponentBuilder
 from rasa.nlu.config import RasaNLUModelConfig, component_config_from_pipeline
-from rasa.nlu.extractors.extractor import EntityExtractor  # pytype: disable=pyi-error
+from rasa.nlu.extractors.extractor import EntityExtractor
 
 from rasa.nlu.persistor import Persistor
 from rasa.shared.nlu.constants import (
@@ -163,13 +161,13 @@ class Trainer:
         self, cfg: RasaNLUModelConfig, component_builder: ComponentBuilder
     ) -> List[Component]:
         """Transform the passed names of the pipeline components into classes."""
-
         pipeline = []
 
         # Transform the passed names of the pipeline components into classes
-        for i in range(len(cfg.pipeline)):
-            component_cfg = cfg.for_component(i)
+        for index, pipeline_component in enumerate(cfg.pipeline):
+            component_cfg = cfg.for_component(index)
             component = component_builder.create_component(component_cfg, cfg)
+            components.validate_component_keys(component, pipeline_component)
             pipeline.append(component)
 
         if not self.skip_validation:
