@@ -8,7 +8,7 @@ from typing import Any, Dict, Optional, Text, Tuple, Union, List, Type
 
 from rasa.shared.nlu.training_data import util
 import rasa.shared.utils.io
-from rasa.nlu.config import InvalidConfigError
+from rasa.shared.exceptions import InvalidConfigException
 from rasa.shared.nlu.training_data.training_data import TrainingData
 from rasa.shared.nlu.training_data.message import Message
 from rasa.nlu.components import Component
@@ -493,12 +493,12 @@ class ResponseSelector(DIETClassifier):
             meta, model_dir, model_metadata, cached_component, **kwargs
         )
         if not meta.get("file"):
-            return model  # pytype: disable=bad-return-type
+            return model
 
         model.responses = meta.get("responses", {})
         model.all_retrieval_intents = meta.get("all_retrieval_intents", [])
 
-        return model  # pytype: disable=bad-return-type
+        return model
 
 
 class DIET2BOW(DIET):
@@ -540,14 +540,16 @@ class DIET2BOW(DIET):
 
 
 class DIET2DIET(DIET):
+    """Diet 2 Diet transformer implementation."""
+
     def _check_data(self) -> None:
         if TEXT not in self.data_signature:
-            raise InvalidConfigError(
+            raise InvalidConfigException(
                 f"No text features specified. "
                 f"Cannot train '{self.__class__.__name__}' model."
             )
         if LABEL not in self.data_signature:
-            raise InvalidConfigError(
+            raise InvalidConfigException(
                 f"No label features specified. "
                 f"Cannot train '{self.__class__.__name__}' model."
             )
