@@ -1,10 +1,12 @@
 TEXT = "text"
 INTENT = "intent"
 RESPONSE = "response"
+RESPONSE_SELECTOR = "response_selector"
 INTENT_RESPONSE_KEY = "intent_response_key"
 ACTION_TEXT = "action_text"
 ACTION_NAME = "action_name"
 INTENT_NAME_KEY = "name"
+FULL_RETRIEVAL_INTENT_NAME_KEY = "full_retrieval_intent_name"
 METADATA = "metadata"
 METADATA_INTENT = "intent"
 METADATA_EXAMPLE = "example"
@@ -33,3 +35,6 @@ ENTITY_ATTRIBUTE_VALUE = "value"
 ENTITY_ATTRIBUTE_START = "start"
 ENTITY_ATTRIBUTE_END = "end"
 NO_ENTITY_TAG = "O"
+SPLIT_ENTITIES_BY_COMMA = "split_entities_by_comma"
+SPLIT_ENTITIES_BY_COMMA_DEFAULT_VALUE = True
+SINGLE_ENTITY_ALLOWED_INTERLEAVING_CHARSET = {".", ",", " ", ";"}
