@@ -1,24 +1,21 @@
+import asyncio
 import logging
 import os
 import shutil
 import warnings
 from types import TracebackType
-from typing import Any, Callable, Dict, List, Optional, Text, Type
+from typing import Any, Coroutine, Dict, List, Optional, Text, Type, TypeVar
 
 import rasa.core.utils
 import rasa.utils.io
-from rasa.cli import utils
-from rasa.cli.utils import bcolors
-from rasa.constants import (
-    DEFAULT_LOG_LEVEL,
-    DEFAULT_LOG_LEVEL_LIBRARIES,
-    ENV_LOG_LEVEL,
-    ENV_LOG_LEVEL_LIBRARIES,
-    GLOBAL_USER_CONFIG_PATH,
-)
+from rasa.constants import DEFAULT_LOG_LEVEL_LIBRARIES, ENV_LOG_LEVEL_LIBRARIES
+from rasa.shared.constants import DEFAULT_LOG_LEVEL, ENV_LOG_LEVEL
+import rasa.shared.utils.io
 
 logger = logging.getLogger(__name__)
 
+T = TypeVar("T")
+
 
 class TempDirectoryPath(str):
     """Represents a path to an temporary directory. When used as a context
@@ -39,18 +36,17 @@ class TempDirectoryPath(str):
             shutil.rmtree(self)
 
 
-def arguments_of(func: Callable) -> List[Text]:
-    """Return the parameters of the function `func` as a list of names."""
-    import inspect
+def read_global_config(path: Text) -> Dict[Text, Any]:
+    """Read global Rasa configuration.
 
-    return list(inspect.signature(func).parameters.keys())
-
-
-def read_global_config() -> Dict[Text, Any]:
-    """Read global Rasa configuration."""
+    Args:
+        path: Path to the configuration
+    Returns:
+        The global configuration
+    """
     # noinspection PyBroadException
     try:
-        return rasa.utils.io.read_config_file(GLOBAL_USER_CONFIG_PATH)
+        return rasa.shared.utils.io.read_config_file(path)
     except Exception:
         # if things go south we pretend there is no config
         return {}
@@ -60,7 +56,6 @@ def set_log_level(log_level: Optional[int] = None):
     """Set log level of Rasa and Tensorflow either to the provided log level or
     to the log level specified in the environment variable 'LOG_LEVEL'. If none is set
     a default log level will be used."""
-    import logging
 
     if not log_level:
         log_level = os.environ.get(ENV_LOG_LEVEL, DEFAULT_LOG_LEVEL)
@@ -194,84 +189,34 @@ def obtain_verbosity() -> int:
     return verbosity
 
 
-def is_logging_disabled() -> bool:
-    """Returns true, if log level is set to WARNING or ERROR, false otherwise."""
-    log_level = os.environ.get(ENV_LOG_LEVEL, DEFAULT_LOG_LEVEL)
-
-    return log_level == "ERROR" or log_level == "WARNING"
-
-
 def sort_list_of_dicts_by_first_key(dicts: List[Dict]) -> List[Dict]:
     """Sorts a list of dictionaries by their first key."""
     return sorted(dicts, key=lambda d: list(d.keys())[0])
 
 
-# noinspection PyUnresolvedReferences
-def class_from_module_path(
-    module_path: Text, lookup_path: Optional[Text] = None
-) -> Any:
-    """Given the module name and path of a class, tries to retrieve the class.
-
-    The loaded class can be used to instantiate new objects. """
-    import importlib
-
-    # load the module, will raise ImportError if module cannot be loaded
-    if "." in module_path:
-        module_name, _, class_name = module_path.rpartition(".")
-        m = importlib.import_module(module_name)
-        # get the class, will raise AttributeError if class cannot be found
-        return getattr(m, class_name)
-    else:
-        module = globals().get(module_path, locals().get(module_path))
-        if module is not None:
-            return module
-
-        if lookup_path:
-            # last resort: try to import the class from the lookup path
-            m = importlib.import_module(lookup_path)
-            return getattr(m, module_path)
-        else:
-            raise ImportError(f"Cannot retrieve class from path {module_path}.")
-
-
-def minimal_kwargs(
-    kwargs: Dict[Text, Any], func: Callable, excluded_keys: Optional[List] = None
-) -> Dict[Text, Any]:
-    """Returns only the kwargs which are required by a function. Keys, contained in
-    the exception list, are not included.
+def write_global_config_value(name: Text, value: Any) -> bool:
+    """Read global Rasa configuration.
 
     Args:
-        kwargs: All available kwargs.
-        func: The function which should be called.
-        excluded_keys: Keys to exclude from the result.
+        name: Name of the configuration key
+        value: Value the configuration key should be set to
 
     Returns:
-        Subset of kwargs which are accepted by `func`.
-
+        `True` if the operation was successful.
     """
-
-    excluded_keys = excluded_keys or []
-
-    possible_arguments = arguments_of(func)
-
-    return {
-        k: v
-        for k, v in kwargs.items()
-        if k in possible_arguments and k not in excluded_keys
-    }
-
-
-def write_global_config_value(name: Text, value: Any) -> None:
-    """Read global Rasa configuration."""
-
+    # need to use `rasa.constants.GLOBAL_USER_CONFIG_PATH` to allow patching
+    # in tests
+    config_path = rasa.constants.GLOBAL_USER_CONFIG_PATH
     try:
-        os.makedirs(os.path.dirname(GLOBAL_USER_CONFIG_PATH), exist_ok=True)
+        os.makedirs(os.path.dirname(config_path), exist_ok=True)
 
-        c = read_global_config()
+        c = read_global_config(config_path)
         c[name] = value
-        rasa.core.utils.dump_obj_as_yaml_to_file(GLOBAL_USER_CONFIG_PATH, c)
+        rasa.shared.utils.io.write_yaml(c, rasa.constants.GLOBAL_USER_CONFIG_PATH)
+        return True
     except Exception as e:
         logger.warning(f"Failed to write global config. Error: {e}. Skipping.")
+        return False
 
 
 def read_global_config_value(name: Text, unavailable_ok: bool = True) -> Any:
@@ -283,10 +228,14 @@ def read_global_config_value(name: Text, unavailable_ok: bool = True) -> Any:
         else:
             raise ValueError(f"Configuration '{name}' key not found.")
 
-    if not os.path.exists(GLOBAL_USER_CONFIG_PATH):
+    # need to use `rasa.constants.GLOBAL_USER_CONFIG_PATH` to allow patching
+    # in tests
+    config_path = rasa.constants.GLOBAL_USER_CONFIG_PATH
+
+    if not os.path.exists(config_path):
         return not_found()
 
-    c = read_global_config()
+    c = read_global_config(config_path)
 
     if name in c:
         return c[name]
@@ -294,91 +243,23 @@ def read_global_config_value(name: Text, unavailable_ok: bool = True) -> Any:
         return not_found()
 
 
-def mark_as_experimental_feature(feature_name: Text) -> None:
-    """Warns users that they are using an experimental feature."""
+def update_existing_keys(
+    original: Dict[Any, Any], updates: Dict[Any, Any]
+) -> Dict[Any, Any]:
+    """Iterate through all the updates and update a value in the original dictionary.
 
-    logger.warning(
-        f"The {feature_name} is currently experimental and might change or be "
-        "removed in the future 🔬 Please share your feedback on it in the "
-        "forum (https://forum.rasa.com) to help us make this feature "
-        "ready for production."
-    )
+    If the updates contain a key that is not present in the original dict, it will
+    be ignored."""
 
-
-def lazy_property(function: Callable) -> Any:
-    """Allows to avoid recomputing a property over and over.
-
-    The result gets stored in a local var. Computation of the property
-    will happen once, on the first call of the property. All
-    succeeding calls will use the value stored in the private property."""
-
-    attr_name = "_lazy_" + function.__name__
-
-    @property
-    def _lazyprop(self):
-        if not hasattr(self, attr_name):
-            setattr(self, attr_name, function(self))
-        return getattr(self, attr_name)
-
-    return _lazyprop
-
-
-def raise_warning(
-    message: Text,
-    category: Optional[Type[Warning]] = None,
-    docs: Optional[Text] = None,
-    **kwargs: Any,
-) -> None:
-    """Emit a `warnings.warn` with sensible defaults and a colored warning msg."""
-
-    original_formatter = warnings.formatwarning
-
-    def should_show_source_line() -> bool:
-        if "stacklevel" not in kwargs:
-            if category == UserWarning or category is None:
-                return False
-            if category == FutureWarning:
-                return False
-        return True
-
-    def formatwarning(
-        message: Text,
-        category: Optional[Type[Warning]],
-        filename: Text,
-        lineno: Optional[int],
-        line: Optional[Text] = None,
-    ):
-        """Function to format a warning the standard way."""
-
-        if not should_show_source_line():
-            if docs:
-                line = f"More info at {docs}"
-            else:
-                line = ""
-
-        formatted_message = original_formatter(
-            message, category, filename, lineno, line
-        )
-        return utils.wrap_with_color(formatted_message, color=bcolors.WARNING)
-
-    if "stacklevel" not in kwargs:
-        # try to set useful defaults for the most common warning categories
-        if category == DeprecationWarning:
-            kwargs["stacklevel"] = 3
-        elif category == UserWarning:
-            kwargs["stacklevel"] = 2
-        elif category == FutureWarning:
-            kwargs["stacklevel"] = 3
-
-    warnings.formatwarning = formatwarning
-    warnings.warn(message, category=category, **kwargs)
-    warnings.formatwarning = original_formatter
+    updated = original.copy()
+    for k, v in updates.items():
+        if k in updated:
+            updated[k] = v
+    return updated
 
 
 class RepeatedLogFilter(logging.Filter):
-    """
-    Filter repeated log records.
-    """
+    """Filter repeated log records."""
 
     last_log = None
 
@@ -394,3 +275,40 @@ class RepeatedLogFilter(logging.Filter):
             self.last_log = current_log
             return True
         return False
+
+
+def run_in_loop(
+    f: Coroutine[Any, Any, T], loop: Optional[asyncio.AbstractEventLoop] = None
+) -> T:
+    """Execute the awaitable in the passed loop.
+
+    If no loop is passed, the currently existing one is used or a new one is created
+    if no loop has been started in the current context.
+
+    After the awaitable is finished, all remaining tasks on the loop will be
+    awaited as well (background tasks).
+
+    WARNING: don't use this if there are never ending background tasks scheduled.
+        in this case, this function will never return.
+
+    Args:
+       f: function to execute
+       loop: loop to use for the execution
+
+    Returns:
+        return value from the function
+    """
+
+    if loop is None:
+        try:
+            loop = asyncio.get_event_loop()
+        except RuntimeError:
+            loop = asyncio.new_event_loop()
+            asyncio.set_event_loop(loop)
+    result = loop.run_until_complete(f)
+
+    # Let's also finish all running tasks:
+    pending = asyncio.Task.all_tasks()
+    loop.run_until_complete(asyncio.gather(*pending))
+
+    return result
